/*
    Spectroscopic Toolkit by Pieter Suurmond, december 2, 2002.

    Under UNIX, one may use the provided makefile to compile ST.  Under MacOS, 
    Windows, DOS, etc., put at least the following sourcefiles in the compiler's
    C-project:
                    ST.c
                    ST_aiff.c
                    ST_wavelets.c
                    ST_score.c
                    gt/gt.c
                    pconv/ST_more.pbin.c

    And one or more of the composition files in subdirectory comp:

        c00_example.c,       (Some examples to demonstrate use of addWAVELET(),
                              not using any spectroscopic data yet.)
        c01_twoWeeks.c,      (7 alchemistic metals + ions.)
        c02_smear.c,         (Pseudo-glissandi, ascending.)
        c03_H.c,             (Hydrogen.)
        c04_Mendelejev.c     (Mendelejev progression.)
        cXX_xxxx.c           (Or any other composition source.)

    Apart from ST_c00_example.c, all ST_cXX_xxxx.c files use the spectroscopic
    database file of Robert L. Kurucz. So except from ST_c00_example.c, all 
    compostion sources expect a correct version of gfall.pbin to be there,
    in subdirectory pconv
    
    In csh under IRIX, this executable can be run nicely in the background.
    On a remote machine, like this:
                                        nice +20 ST >zST.stdout &
*/

#include <stdio.h>                  /* Standard headers. */
#include <stdlib.h>
#include <math.h>

#include "ST_wavelets.h"            /* For ENGINE.  */
#include "ST_score.h"               /* To produce graphic scores in JPG format.           */
#include "c00.h"                    /* All prototypes of the composing functions you use. */

/*----------------------------------------------------------------------------------------*/
int main(/*int n, char **args*/)                    /* No arguments (yet).                */
{
    char*   version     = "ST1.62";                 /* UPDATE whenever you change things. */
    char*   logfileName = "zST.log";                /* Specify pathname of logfile here.  */
    FILE*   logfileMSG  = fopen(logfileName, "wt"); /* Try to open (overwrite).           */
    FILE*   consoleMSG  = stdout;                   /* One may change this to NULL to     */
    short   e = 0;                                  /* surpress messaging to terminal.    */
    ENGINE  E;

    if (logfileMSG) /* Below, we tust it not to be NULL. */
        {
        fprintf(logfileMSG, "------ LOG OUTPUT FROM '%s' ------\n", version);
        if (consoleMSG)
            fprintf(consoleMSG, "\n%s writing to logfile '%s'.\n", version, logfileName);
        /*------------------------------------------------ INIT DSP: --------------------*/
        E = newENGINE(44100L, 8192L, consoleMSG);       /* Samplerate and blocksize.     */
        if (E)
            {
#if 0       /* You don't need to do this all the time (see copies in html subdirectory). */
            visualiseTables(E, 250, 160, consoleMSG);   /* Engine, width, height, FILE*. */
#endif      /*------------------------------ COMPOSE: -----------------------------------*/

            e = c00_example1(E, logfileMSG);    /* JUST A SMALL EXAMPLE-COMPOSITION. */
                                                /* See subdirectory 'comp' for more. */
            if (e)
                { if (consoleMSG) fprintf(consoleMSG, "Composing failure = %d!\n", e); }
            else
                {
                fflush(logfileMSG);  /* Just handy. */
                /*--------------------- GENERATE GRAPHIC SCORES: ------------------------*/
                /*            engine  filename:     width: height: envTrack: title: msg: */
                e = scoreWAVELETS(E, "zSTbig.jpg",  1200,  1200,   0, version, consoleMSG);
                if (e) fprintf(logfileMSG, "scoreWAVELETS() = %d!\n", e);
                
                e = scoreWAVELETS(E, "zSTsmall.jpg", 600,  600,    1, version, consoleMSG);
                if (e) fprintf(logfileMSG, "scoreWAVELETS() = %d!\n", e);
                /*--------------------- RENDER AUDIO TO AIFF: ---------------------------*/
                /*               engine: filename:   bits:  MSG:        releaseMSG:      */
                e = playWAVELETS(E,      "zST.aiff", 24,    logfileMSG, consoleMSG);
                }
            releaseENGINE(&E);             /* Frees memory and wavelets if any remained. */
            }
        fprintf(logfileMSG, "------ END OF LOG OUTPUT FROM '%s' ------\n", version);
        fclose(logfileMSG);
        }
    else if (consoleMSG)
        fprintf(consoleMSG, "Unable to create (overwrite) logfile '%s'!\n", logfileName);
    return (int)e;                          /* exit(0); */
}
