/*
	Spectroscopic Toolkit version 1.40 by Pieter Suurmond, november 11, 2002.
	Tiny audiofile tool, providing two ways of generating AIFF files: one in which
	filesize is known in advance, and the other one where eventual filesize is not 
	known in advance.
*/

#define kAIFFWRITE_max_frames    	79380000	/* Default maximum length (30 min  */
    											/* @44.1kHz 16 bit stereo, 303 MB) */
#define kAIFFWRITE_min_channels    	1
#define kAIFFWRITE_max_channels    	2

#define kAIFFWRITE_min_samplerate  	1L		/* Tolerant regarding exotic samplerates.  */
#define kAIFFWRITE_max_samplerate  	144000L


typedef struct
{
    FILE            *fp;        	/* File stream pointer.                            */
    unsigned long   frames;     	/* Total number of frames    (4 byte big-endian).  */
	short			rewrite;		/* Set if frames was 0 in call to openAIFFWRITE(). */

    unsigned short  channels;   	/* Number of sound channels  (2 byte big-endian).  */
    unsigned short  bits;       	/* Number of bits            (2 byte big-endian).  */
    long            rate;       	/* Sampling rate in cycles per second.             */        
}   AUDIOFILE;


short	openAIFFWRITE (AUDIOFILE** 		A,
					   char*			filename,
					   long         	samplerate,
					   unsigned long	frames,
    				   unsigned short	channels,
    				   unsigned short	bits);
/*
	Tries to create a new AIFF file with name filename. If it succeeds, zero is 
	returned and argument A receives an AUDIOFILE object.
	A negative error number is returned when handle A is NULL, filename is NULL 
	or empty, ...
	Calling openAIFFWRITE() with zero frames, the header of the AIFF file will
	be rewritten in any successive closeAIFFWRITE() call.

	When openAIFFWRITE() is called with nonzero frames, this is assumed to be the 
	right number of frames and header info will not be rewritten in a successive 
	call to closeAIFFWRITE().
*/


short 	dumpAIFFWRITE (AUDIOFILE* A, void* audio, unsigned long frames);


void	printAIFFWRITE (AUDIOFILE* A, FILE* to);
/*
	No problem when one or both arguments are NULL.
*/


short 	closeAIFFWRITE (AUDIOFILE** A);
/*
	Rewrites header info only if the object was previously created with zero frames,
	closes the file, releases memory and clears the reference to the AUDIOFILE object.

	Expects channels, bits, rate and frames to be set in struct A.
	Returns 0 on success.
	Returns a negative error number on failure, when error <= 100, rewriting of
	header failed.
*/
