/*
	Spectroscopic Toolkit version 1.60 by Pieter Suurmond, december 1, 2002.
*/

#include <stdio.h>        		/* Standard headers. */
#include <stdlib.h>
#include <math.h>

#include "ST_wavelets.h"		/* Prototype addWAVELET(). */
#include "c00.h"                /* To check prototypes.    */

/*----------------------------------------------------------------------------------------*/
short c00_example0(ENGINE E, FILE* msg) /* Just a small test/example, not using database. */
{                                       /* Simply puts 4 wavelets in the linked list.     */

                            /* Freq:   Lin-amp:  Panning:   Top:    Dur:   */
                            /* Hertz   -1..+1    -1..+1     s       s      */

    if (addWAVELET(E, msg,     120.0,  -1.00,    -1.00,     4.00,   0.40, kENV_Gaussian, 1))
        return(1);          /* Try amp=+1.00, which causes overflow (as designed). At 24  */
                            /* and 16 bit output-size, -1.0 for such a short time (0.4 s) */
                            /* does NOT give underflow, because -1 is allowed, +1 isn't.  */ 
    if (addWAVELET(E, msg,     130.0,  -1.00,    +1.00,     8.00,   0.40, kENV_Gaussian, 1))
        return(1);
    if (addWAVELET(E, msg,     440.0,   0.10,    -0.75,    12.00,   2.00, kENV_Gaussian, 1))
        return(1);
    if (addWAVELET(E, msg,    8800.0,   0.20,    +0.25,    14.00,   2.00, kENV_Gaussian, 1))
        return(1);

    return(0);
}

/*---------------------------------------------------------------------------------------------*/
short c00_example1(ENGINE E, FILE* msg) /* Another test/example, again not using any database. */
{                                       /* Puts some overtone-wavelets in the linked list.     */
    short   n, m;
    double  d, s;

    for (m=1; m<=8; m++)
        for (n=m; n<=64; n++)                           /* n is harmonics-number, fund = 60Hz. */
            {
            d = 1.0 / (double)n;                        /* Higher harmonics will be shorter.   */
            s = 12.0 +                                  /* Start-inflection-point.             */
                (2.0*(double)m) + 0.1 * (double)(n-6*m);
            if (addWAVELET(E, msg,
                           (double)n * 60.0,            /* 60 Hertz as fundamental.            */
                            0.4 * d,                    /* amplitude decreasing with time.     */
                            -0.5 + (double)(n-1)/71.0,  /* pan from left to right.             */
                            s + 0.5 * d,                /* top (@ start-infl + half duration). */
                            3.0 * d,                    /* duration in seconds.                */
							kENV_Gaussian,
							1))							/* 1 means sync with other oscillators.*/

                {
                printf("Illegal wavelet!\n");
                if (msg) fprintf(msg, "\"c00_example1()\" stopped!\n"); /* Optional messaging. */
                return(1);
                }
            }
    return(0);
}

/*---------------------------------------------------------------------------------------*/
short c00_phaseTest(ENGINE E, FILE* msg)    /* Connecting overlapping in-phase wavelets  */
{                                           /* with exactly the same frequency.          */

                            /* Freq:   Lin-amp:  Panning:   Top:    Dur:   */
                            /* Hertz   -1..+1    -1..+1     s       s      */

	/*-------------------------- Connecting at inflection points @ 0.6065: QUITE SMOOTH */
    if (addWAVELET(E, msg,     880.0,   0.50,    +1.00,     4.00,   0.20, kENV_Gaussian, 1))
        return(1);
    if (addWAVELET(E, msg,     880.0,   0.50,    +1.00,     4.20,   0.20, kENV_Gaussian, 1))
        return(1);
    if (addWAVELET(E, msg,     880.0,   0.50,    +1.00,     4.40,   0.20, kENV_Gaussian, 1))
        return(1);
    if (addWAVELET(E, msg,     880.0,   0.50,    +1.00,     4.60,   0.20, kENV_Gaussian, 1))
        return(1);
	if (addWAVELET(E, msg,     880.0,   0.50,    +1.00,     4.80,   0.20, kENV_Gaussian, 1))
        return(1);
	if (addWAVELET(E, msg,     880.0,   0.50,    +1.00,     5.00,   0.20, kENV_Gaussian, 1))
        return(1);
	if (addWAVELET(E, msg,     880.0,   0.50,    +1.00,     5.20,   0.20, kENV_Gaussian, 1))
        return(1);
	if (addWAVELET(E, msg,     880.0,   0.50,    +1.00,     5.40,   0.20, kENV_Gaussian, 1))
        return(1);

	/*------------------------- Connecting beyond inflection points @ 0.5: BUBBLES */
	/* -6 dB points at TOP-0.588705011*DUR and TOP+0.588705011*DUR.        */
    if (addWAVELET(E, msg,     866.0,   0.50,    -1.00,     8.00,     0.40, kENV_Gaussian, 1))
        return(1);
    if (addWAVELET(E, msg,     866.0,   0.50,    -1.00,     8.470964, 0.40, kENV_Gaussian, 1))
        return(1);
    if (addWAVELET(E, msg,     866.0,   0.50,    -1.00,     8.941928, 0.40, kENV_Gaussian, 1))
        return(1);
    if (addWAVELET(E, msg,     866.0,   0.50,    -1.00,     9.412892, 0.40, kENV_Gaussian, 1))
        return(1);

	/*-------------------------- Connecting at inflection points @ 0.6065: A GONG */
    if (addWAVELET(E, msg,     800.0,   0.50,    +1.00,    12.00,   0.10, kENV_Gaussian, 1))
        return(1);										/* 12.05 */
    if (addWAVELET(E, msg,     800.0,   0.40,    +1.00,    12.15,   0.20, kENV_Gaussian, 1))
        return(1);										/* 12.25 */
    if (addWAVELET(E, msg,     800.0,   0.30,    +1.00,    12.45,   0.40, kENV_Gaussian, 1))
        return(1);										/* 12.65 */
    if (addWAVELET(E, msg,     800.0,   0.20,    +1.00,    13.05,   0.80, kENV_Gaussian, 1))
        return(1);										/* 13.45 */
	if (addWAVELET(E, msg,     800.0,   0.10,    +1.00,    14.25,   1.60, kENV_Gaussian, 1))
        return(1);										/* 15.05 */

    return(0);
}


/*---------------------------------------------------------------------------------------*/
short c00_expTest(ENGINE E, FILE* msg)      /* Connecting overlapping in-phase wavelets  */
{	                                        /* with exactly the same frequency.          */

                            /* Freq:   Lin-amp:  Panning:   Top:    Dur:   */
                            /* Hertz   -1..+1    -1..+1     s       s      */

    if (addWAVELET(E, msg,     440.0,   0.50,    +1.00,     16.33,   1.00, kENV_ExpRise, 0))
        return(1);
    if (addWAVELET(E, msg,     880.0,  -1.00,    -1.00,     20.21,   0.30, kENV_ExpRise, 0))
        return(1);
    if (addWAVELET(E, msg,     500.0,   0.50,    +1.00,     31.18,   1.98, kENV_ExpRise, 0))
        return(1);

    if (addWAVELET(E, msg,     940.0,   0.50,    -1.00,     39.73,   0.033, kENV_ExpRise, 0))
        return(1);
    if (addWAVELET(E, msg,    1080.0,  -1.00,    +1.00,     41.16,   0.01, kENV_ExpRise, 0))
        return(1);
    if (addWAVELET(E, msg,     150.0,   0.75,    -1.00,     50.582,  2.70, kENV_ExpRise, 0))
        return(1);

    if (addWAVELET(E, msg,     150.0,   0.75,    -1.00,     50.582,  2.70, kENV_ExpDecay, 0))
        return(1);


    return(0);
}

