/*
	Spectroscopic Toolkit version 1.40 by Pieter Suurmond, november 11, 2002.
	Using the binary database-file "gfall.pbin", produced by my "pconv"-program 
	that reads Robert Kurucz's latest data file.
 	ST translates spectroscopic (i.e. quantumphysical) data to AIFF-file.
*/

#include <stdio.h>        		/* Standard headers. */
#include <stdlib.h>
#include <math.h>

#include "ST_wavelets.h"		/* Prototype addWAVELET().  */

#include "gfall.pbin.h"        	/* Defines structure of the binary database-file,    */
                              	/* same header as included in the 'pconv'-program.   */
#include "ST_more.pbin.h"		/* To extract some more out of the database.         */
								/* But also path to the database. 					 */


typedef struct          /* Same as in c01_twoWeeks(). */
{
    int   element;
    double  overalAmplitude;
    /*-------------------- diagnostic: */
    double  loFreq,     /* AUDIO-freq. */
            hiFreq,
            loAmp,      /* Hi-Hertz(!), not audio-amplitude yet. */
            hiAmp;
    double  durA, durB; /* Slope & offset. Lowest freq->1; highest freq->0.5. */
    /*-------------------- RGB-values for video: */
    float   RGBaccu[3];
}   week;               /* For duration- and amplitude-compensation. */


/*-------------------------------------------------------------------------------------------*/
short c02_weekSmear(ENGINE* E, FILE* msg)               /* msg = NULL, stdout or logfile.    */
{                                                       /* Just 1 metal a time / time-smear. */
    FILE            *pbinDatabaseFP;
    pieterPacked    pbinBuff;
    derivedData     more;                               /* See ST.gfall.pbin.c. */
    unsigned long   linesRead = 0, linesUsed = 0, linesSkipped = 0;
    short           e = 0, n;
    short           atomicNumber, ionizationLevel;      /* "week"-struct declared in ST.compose00.c. */
    double          audioHertz, a, dur, p, inflectStart, d;
    week            sections[7] = { /* Atomic.num: OveralAmp:   (statistics:) */
                    /* Au */            79,         3.20,       9.0e99, 0.0,    9.0e99, 0.0, 0.0, 0.0,   0.0, 0.0, 0.0,
                    
                    /* Ag */            47,         3.30,       9.0e99, 0.0,    9.0e99, 0.0, 0.0, 0.0,   0.0, 0.0, 0.0,
                    /* Fe */            26,/* Fe2+*/0.10,       9.0e99, 0.0,    9.0e99, 0.0, 0.0, 0.0,   0.0, 0.0, 0.0,
                    
                    /* Hg */            80,         1.00,       9.0e99, 0.0,    9.0e99, 0.0, 0.0, 0.0,   0.0, 0.0, 0.0,
                                                /* one Hg comp has to be weakened! */
                    /* Sn */            50,         1.00,       9.0e99, 0.0,    9.0e99, 0.0, 0.0, 0.0,   0.0, 0.0, 0.0,
                    /* Cu */            29,         0.88,       9.0e99, 0.0,    9.0e99, 0.0, 0.0, 0.0,   0.0, 0.0, 0.0,
                    
                    /* Pb */            82,         1.40,       9.0e99, 0.0,    9.0e99, 0.0, 0.0, 0.0,   0.0, 0.0, 0.0 };
    /*------------------------------------------------------------ Frequency & time transpositions:  */
    double          dayTime = 24.0 * 3600.0 / 4096.0;           /* Secs per day, transposed 12 octs. */
    double          transpositionRatio = pow(2.0, -40.0);       /* 40 octaves down.                  */
    /*------------------------------------------------------------ Smoother frequency windowing:     */
    double          lowestFreq = 20.0;                          /* (trapeziod instead of brickwall)  */
    double          aboveLowestFreq = 1.25 * lowestFreq;        /* Rise to 1 within 1 major third.   */
    double          aboveHertz = aboveLowestFreq - lowestFreq;
    double          nyquist = 0.5 * (double)E->samplerate;
    double          belowNyquist = nyquist / 1.2;               /* Fall to 0 within 1 minor third.   */
    double          belowHertz = nyquist - belowNyquist;
    /*------------------------------------------------------------ Open binary version of Kurucz's file. */
    if (pbinDatabaseFP = fopen(kDbase, "rb"))
        {
        /*------------------------------ CYCLE #1 (analysis) ------------------*/
        printf("Analyzing database \"%s\"...\n", kDbase);
        while (fread(&pbinBuff, sizeof(pbinBuff), 1, pbinDatabaseFP))
            {
            atomicNumber = pbinBuff.ELEM / 100;
            ionizationLevel = pbinBuff.ELEM - (100 * atomicNumber);
            n = 0;
            while ((sections[n].element != atomicNumber) && (n < 7))    /* See if it's in our selection. */
                n++;
            if ((n < 7) && (ionizationLevel==2))                          /* +2 IONS. */
                {
                moreLineData(&pbinBuff, &more);                         /* Calc some more data. */
                audioHertz = more.frequency * transpositionRatio;
                /* Exclude out-of-range-freqs from analysis. */
                /* There could slip through some freqs that are too weak! */
                if ((audioHertz > lowestFreq) && (audioHertz < nyquist))
                    {
                    if (audioHertz < sections[n].loFreq)
                        sections[n].loFreq = audioHertz;
                    if (audioHertz > sections[n].hiFreq)
                        sections[n].hiFreq = audioHertz;
                    if (more.Avalue < sections[n].loAmp)
                        sections[n].loAmp = more.Avalue;
                    if (more.Avalue > sections[n].hiAmp)
                        sections[n].hiAmp = more.Avalue;
                    }
                }
            }
        /*--------------------- Print file-statistics for the selected metals: -------*/
        for (n=0; n<7; n++)  /* AT CYCLE #2 these minima & maxima are used to scale.  */
            {
            if (msg)
                fprintf(msg, "-------- element %d: --------\n\
loAmp = %.8f; hiAmp = %.8f; RATIO=%.2f.\nloFreq = %.6f; hiFreq = %.6f; RATIO=%.2f.\n",
                sections[n].element,
                sections[n].loAmp, sections[n].hiAmp, sections[n].hiAmp / sections[n].loAmp,
                sections[n].loFreq, sections[n].hiFreq, sections[n].hiFreq / sections[n].loFreq);
            /*---------------- Linear duration compensation function: ------------------------------*/
            /* d(loF) = 1           1 = a loF + b          a = 0.5 / (loF - hiF)                    */
            /* d(hiF) = 0.5       0.5 = a hiF + b  -     0.5 = (0.5 / (loF - hiF)) hiF + b          */
            /* d(F) = a F + b     -------------------      b = 0.5 - (0.5 hiF / (loF - hiF))        */
            /*                    0.5 = a (loF - hiF)                                               */
            d = sections[n].loFreq - sections[n].hiFreq;
            sections[n].durA = 0.5 / d;                                 /* A=slope and B=offset. */
            sections[n].durB = 0.5 - (0.5 * sections[n].hiFreq / d);
            /*---------------- Linear bass-boost-filter-function (per element): --------------------*/
            /* a(loF) = 1             SAME d as with duration may be used!                          */
            /* d(hiF) = 0.5 (-6dB)  0.5 = a hiF + b  -                                              */
            }
        /*------------------------------ CYCLE #2 ------------------*/
        rewind(pbinDatabaseFP);
        printf("Posting wavelets...\n");
        if (msg)
            {
            fprintf(msg, "1/transpositionRatio = %.1f.\n", 1.0 / transpositionRatio);
            fprintf(msg, "dayTime = %.6f s.\n", dayTime);
            }
        while (fread(&pbinBuff, sizeof(pbinBuff), 1, pbinDatabaseFP))
            {
            /*---------------------------------------------------- Get Atomic number and ionization level. */
            atomicNumber = pbinBuff.ELEM / 100;
            ionizationLevel = pbinBuff.ELEM - (100 * atomicNumber);
            /*---------------------------------------------------- ELEMENT/ION - selector: */
            n = 0;
            while ((sections[n].element != atomicNumber) && (n < 7))     /* See if it's in our selection. */
                n++;                                                     /* Lookup our "n"-index.         */
            
            /********** (Au=0) SELECT YOUR METAL HERE !!!! *********************/
            if ((n == 2) && (ionizationLevel==2))  /* Geioniseerd Fe+2. 0=Au, 1=Ag, etc. */
                {
                moreLineData(&pbinBuff, &more);                 /* Calc some more data. */
                /*----------------------------------------------------------------------*/
                audioHertz = more.frequency * transpositionRatio;
                if ((audioHertz > lowestFreq) && (audioHertz < nyquist))
                    {
                    /*------------------ Less brickwall-shaped filtering: --------------*/
                    if (audioHertz < aboveLowestFreq)
                        a = (audioHertz - lowestFreq) / aboveHertz; /* Rise to 1 within 1 minor third. */
                    else if (audioHertz > belowNyquist)
                        a = (nyquist - audioHertz) / belowHertz;    /* Fall to 0 within 1 major third. */
                    else
                        a = 1.0;
                    /*---------------- Frequency-dependant Compensation per element: ---------------*/
                    /* Only lowest freq of one metal (section) gets unit-duration and unit-amplit., */
                    /* all higher frequencies sound shorter.  loFreq -> 1,  hiFreq -> 0.5.          */
                    d = (sections[n].durA * audioHertz) + sections[n].durB;
                    /*------------------ Bass boost-filter (per element): --------------*/
                    a *= d;           /* Highest frequency per element -6.021 dB.       */
                    /* ----------------- Amplitude: ------------------------------------*/
                    /* Compress amplitude-range (per section) by approx 4th-power-root. */
                    a *= sections[n].overalAmplitude * pow(1.90e-13 * more.Avalue, 0.26);
                    /*-------------------- Amplitude treshold (23 bit) -----------------*/
                    if (a >= 1.192093E-7)
                        {
                        /*------------------------ Panning: ---------------------------------------*/
                        if (pbinBuff.LOW.J == pbinBuff.UPP.J)
                            p = 0.0;
                        else if (pbinBuff.LOW.J > pbinBuff.UPP.J)   /* Depend upon quantum-states. */
                            p = +0.5;
                        else
                            p = -0.5;
                        if (n & 1)       /* Alternate between left and right channel.       */
                            p += 0.118;  /* Odd ones (Ag, Hg, Cu) slighty to the right.     */
                        else
                            p -= 0.118;  /* Even ones (Au, Fe, Sn, Pb) slighty to the left. */
                        /*---------------- Duration & starttime: ---------------------------*/
                        /* Only lowest freq of one metal (section) gets unit-duration       */
                        dur = 4.0 * d;              /* 4.0 * d instead of dayTime * d   */
                        inflectStart = 20.0;       /* Must be higher for Fe and Cu of coarse!  */
                        inflectStart += /* ((double)n* dayTime) + */ (dayTime * 2.0 * (1.0 - d));
/* From low to high: */
/*********************  +61 for Au; +15 for Ag.*/
                        if (addWAVELET(E, msg,
              /* Hz  */     audioHertz,
              /* amp */     a,
              /* pan */     p,
              /* top */     inflectStart + (0.5 * dur),     /* All start-inflections together. */
              /* dur */     dur,                           /* Top at inflectstart + DUR/2.    */
			  /* env */		kENV_Gaussian,
			  /* syn */		1))			 /* 1 = sync with other oscillators. */
                            {
                            printf("Illegal wavelet!\n");
                            e = 3;
                            goto clFile;
                            }
                        linesUsed++;
                        }
                    else
                        {
                        linesSkipped++;
                        if (msg) fprintf(msg, "SKIPPED-AMP %d: Hz=%12.6f lggf=%5.2f.\n",
                                              atomicNumber, audioHertz, pbinBuff.LOGGF);
                        }
                    }
                else
                    {
                    linesSkipped++;
                    if (msg) fprintf(msg, "SKIPPED-FRQ %d: Hz=%12.6f lggf=%5.2f.\n",
                                          atomicNumber, audioHertz, pbinBuff.LOGGF);
                    }
                printLineEssential(&pbinBuff, msg);        /* stdout or logFP. */
                printLineDerived(&more, msg);
                }
            linesRead++;
            }
        if (msg)
            fprintf(msg, "   linesUsed = %ld.\nlinesSkipped = %ld.\n", linesUsed, linesSkipped);
        if (linesRead != kKuruczLinesTotal)
            {
            printf("Error reading binary database lines!\n");
            e = 2;
            }
clFile: fclose(pbinDatabaseFP);
        }
    else
        {
        printf("Cannot open binary database-file \"%s\"!\n", kDbase);
        e = 1;
        }
    return(e);
}
