/*
 	Spectroscopic Toolkit version 1.60 by Pieter Suurmond, december 1, 2002.
	Using the binary database-file 'gfall.pbin', produced by the 'pconv'-program
 	that reads Robert Kurucz's latest data file. 
	More Hydrogen studies (based on earlier ST.c03_H).
*/

#include <stdio.h>        		/* Standard headers. */
#include <stdlib.h>
#include <math.h>

#include "ST_wavelets.h"		/* Prototype addWAVELET().  */

#include "gfall.pbin.h"        	/* Defines structure of the binary database-file,    */
                              	/* same header as included in the 'pconv'-program.   */
#include "ST_more.pbin.h"		/* To extract some more out of the database.         */
								/* But also path to the database. 					 */

#include "gt.h"                 /* For gtInitHertzToRGBtables() and gtHertzToRGB(). */
#include "c00.h"

typedef struct          		/* Same as in c01_twoWeeks(). */
{
    int   	element;
    double  overalAmplitude;
    /*-------------------- diagnostic: */
    double  loFreq,     /* AUDIO-freq. */
            hiFreq,
            loAmp,      /* Hi-Hertz(!), not audio-amplitude yet. */
            hiAmp;
    double  durA, durB; /* Slope & offset. Lowest freq->1; highest freq->0.5. */
    /*-------------------- RGB-values for video: */
    float   RGBaccu[3];
}   c05_STATISTICS;    	/* For duration- and amplitude-compensation. */


/*---------------------------------------------------------------------------------------------*/
#define kc03_H 1	              				    /* Creates nice 'hydrogen-atmosphere'      */
short c05_niceHydrogenBirds(ENGINE E, FILE* msg)	/* with birds and crickets...              */
{													/* Decaying structure, starts with lower   */
    FILE            *pbinDatabaseFP;				/* pitches, ends with high pitches.        */
    pieterPacked    pbinBuff;
    derivedData     more;                               /* See ST.gfall.pbin.c. */
    unsigned long   linesRead = 0, linesUsed = 0, linesSkipped = 0;
    short           e = 0, n;
    short           atomicNumber, ionizationLevel;
    double          audioHertz, a, dur, d;
	long			repeat;
    float           RGBreceiver[3];     /* To square our RGB-channels before adding. */
    c05_STATISTICS	sections[kc03_H] = {
                    /* Atomic.num: OveralAmp: (statistics:) */
    /* H */         {  1,          6.00,      9.0e99, 0.0,    9.0e99, 0.0, 0.0, 0.0,  {0.0, 0.0, 0.0} } };
                    /* For He, take 2 and 4.00 as OveralAmp, also add 1 to pbinBuff.UPP.J in the 
                       top-calculation below. (He is not as nice as H.) */

    /*------------------------------------------------------------ Frequency & time transpositions:  */
    double          transpositionRatio = pow(2.0, -40.0);       /* 40 octaves down.                  */
    /*------------------------------------------------------------ Smoother frequency windowing:     */
    double          lowestFreq = 20.0;                          /* (trapeziod instead of brickwall)  */
    double          aboveLowestFreq = 1.25 * lowestFreq;        /* Rise to 1 within 1 major third.   */
    double          aboveHertz = aboveLowestFreq - lowestFreq;
    double          nyquist = 0.5*(double)getSamplerateENGINE(E);
    double          belowNyquist = nyquist / 1.2;               /* Fall to 0 within 1 minor third.   */
    double          belowHertz = nyquist - belowNyquist;
    /*------------------------------------------------------------ Energy stats:     */
    double          ELOWmin = 9999999999999999.9, ELOWmax = 0.0,
                    EUPPmin = 9999999999999999.9, EUPPmax = 0.0;
    double          scaledAvalue, rep;
    /*------------------------------------------------------------ Open binary version of Kurucz's file. */
    pbinDatabaseFP = fopen(kDbase, "rb");
    if (pbinDatabaseFP)
        {
        gtInitHertzToRGBtables();               /* Don't forget this, else it'll all be black or random. */
        /*------------------------------ CYCLE #1 (analysis) ------------------*/
        printf("Analyzing database '%s'...\n", kDbase);
        while (fread(&pbinBuff, sizeof(pbinBuff), 1, pbinDatabaseFP))
            {
            atomicNumber = pbinBuff.ELEM / 100;
            ionizationLevel = pbinBuff.ELEM - (100 * atomicNumber);
            n = 0;                                                      /* See if it's in our selection. */
            while ((sections[n].element != atomicNumber) && (n < kc03_H))
                n++;
            if ((n < kc03_H) && (!ionizationLevel))   /* Skip IONS. */
                {
                moreLineData(&pbinBuff, &more);                         /* Calc some more data. */
                audioHertz = more.frequency * transpositionRatio;
                /* Exclude out-of-range-freqs from analysis. */
                /* There could slip through some freqs that are too weak! */
                if ((audioHertz > lowestFreq) && (audioHertz < nyquist))
                    {
                    if (audioHertz < sections[n].loFreq)
                        sections[n].loFreq = audioHertz;
                    if (audioHertz > sections[n].hiFreq)
                        sections[n].hiFreq = audioHertz;
                    if (more.Avalue < sections[n].loAmp)
                        sections[n].loAmp = more.Avalue;
                    if (more.Avalue > sections[n].hiAmp)
                        sections[n].hiAmp = more.Avalue;
                    
                    if (pbinBuff.LOW.E < ELOWmin)
                        ELOWmin = pbinBuff.LOW.E;
                    if (pbinBuff.LOW.E > ELOWmax)
                        ELOWmax = pbinBuff.LOW.E;
                    if (pbinBuff.UPP.E < EUPPmin)
                        EUPPmin = pbinBuff.UPP.E;
                    if (pbinBuff.UPP.E > EUPPmax)
                        EUPPmax = pbinBuff.UPP.E;
                    if (pbinBuff.LOW.E >= pbinBuff.UPP.E)
                        {
                        if (msg) fprintf(msg, "Energy levels swapped!\n\n");
                        }
                    }
                /*------------- accumulate lines in RGB-accumulators (squared sum): -------------*/
                if ((more.frequency >= 40.0e13) && (more.frequency <= 77.0e13))
                    {                               /* 3.1e-9 not tested yet for H. */
                    gtHertzToRGB((float)more.frequency, 3.1e-9 * more.Avalue, RGBreceiver);
                    sections[n].RGBaccu[0] += RGBreceiver[0] * RGBreceiver[0];
                    sections[n].RGBaccu[1] += RGBreceiver[1] * RGBreceiver[1];
                    sections[n].RGBaccu[2] += RGBreceiver[2] * RGBreceiver[2];
                    }
                }
            }
        /*--------------------- Print statistics for the selected metals: ------------*/
        for (n=0; n<kc03_H; n++)  /* AT CYCLE #2 these minima & maxima are used to scale.  */
            {
            if (msg)
                {
                fprintf(msg, "-------- element %d: --------\n\
loAmp = %.20f; hiAmp = %.8f; Amp-ratio=%.2f.\n\
loFreq = %.6f; hiFreq = %.6f; Freq-ratio=%.2f.\n",
                sections[n].element,
                sections[n].loAmp, sections[n].hiAmp, sections[n].hiAmp / sections[n].loAmp,
                sections[n].loFreq, sections[n].hiFreq, sections[n].hiFreq / sections[n].loFreq);
                fprintf(msg, "ELOWmin = %.20f;  ELOWmax = %.2f.\n\
EUPPmin = %.2f;  EUPPmax = %.2f.\n", ELOWmin, ELOWmax, EUPPmin, EUPPmax);
                }
            /*---------------- Linear duration compensation function: ------------------------------*/
            /* d(loF) = 1           1 = a loF + b          a = 0.5 / (loF - hiF)                    */
            /* d(hiF) = 0.5       0.5 = a hiF + b  -     0.5 = (0.5 / (loF - hiF)) hiF + b          */
            /* d(F) = a F + b     -------------------      b = 0.5 - (0.5 hiF / (loF - hiF))        */
            /*                    0.5 = a (loF - hiF)                                               */
            d = sections[n].loFreq - sections[n].hiFreq;
            sections[n].durA = 0.5 / d;                                    /* A=slope and B=offset. */
            sections[n].durB = 0.5 - (0.5 * sections[n].hiFreq / d);
            /*---------------- Linear bass-boost-filter-function (per element): --------------------*/
            /* SAME "d" as with duration may be used: highest freq (per element) gets -6.021 dB.    */
            }
        /*-------------------------------- VIDEO RENDERING (before posting audio-wavelets) ---------*/
        /* printf("Rendering video...\n"); Comment out these lines to skip video rendering.         */
        /* if (renderVideoFromDatabaseH00(sections, dayTime))      Pass pointer to analysis results */
        /*     { e = 4; goto clFile; }                             and amount of seconds for 1 day. */
        /*-------------------------------- CYCLE #2: Post (audio-)wavelets into linked list. -------*/
        rewind(pbinDatabaseFP);
        printf("Posting wavelets (c05_H())...\n");
        if (msg)
            fprintf(msg, "1/transpositionRatio = %.1f.\n", 1.0 / transpositionRatio);
        while (fread(&pbinBuff, sizeof(pbinBuff), 1, pbinDatabaseFP))
            {
            /*---------------------------------------------------- Get Atomic number and ionization level. */
            atomicNumber = pbinBuff.ELEM / 100;
            ionizationLevel = pbinBuff.ELEM - (100 * atomicNumber);
            /*---------------------------------------------------- ELEMENT / ION - selector: */
            n = 0;                                                      /* See if it's in our selection. */
            while ((sections[n].element != atomicNumber) && (n < kc03_H))
                n++;                                                    /* Lookup our "n"-index.         */
            if ((n < kc03_H) && (!ionizationLevel))   /* Skip ionized metals for now.  */
                {
                moreLineData(&pbinBuff, &more);                 /* Calc some more data. */
                /*----------------------------------------------------------------------*/
                audioHertz = more.frequency * transpositionRatio;
                if ((audioHertz > lowestFreq) && (audioHertz < nyquist))
                    {
                    /*------------------ Less brickwall-shaped filtering: --------------*/
                    if (audioHertz < aboveLowestFreq)
                        a = (audioHertz - lowestFreq) / aboveHertz; /* Rise to 1 within 1 minor third. */
                    else if (audioHertz > belowNyquist)
                        a = (nyquist - audioHertz) / belowHertz;    /* Fall to 0 within 1 major third. */
                    else
                        a = 1.0;
                    /*---------------- Frequency-dependant Compensation per element: ---------------*/
                    /* Only lowest freq of one metal (section) gets unit-duration and unit-amplit., */
                    /* all higher frequencies sound shorter.  loFreq -> 1,  hiFreq -> 0.5.          */
                    d = (sections[n].durA * audioHertz) + sections[n].durB;
                    /*------------------ Bass boost-filter (per element): --------------*/
                    a *= d * d * d;   /* Highest frequency per element -18.021 dB.       */
                    /* ----------------- Amplitude: ------------------------------------*/
                    /* Compress amplitude-range (per section) by approx 4th-power-root. */
                    a *= sections[n].overalAmplitude * pow(0.6e-14 * more.Avalue, 0.26);
                    /*-------------------- Amplitude treshold (23 bit) -----------------*/
                    if (a >= 1.192093E-7)
                        {
                        /*---------------- Number of repetitions:---------------------*/
						scaledAvalue = log(more.Avalue / sections[n].loAmp);		/* 0 .. very high. */
						scaledAvalue /= log(sections[n].hiAmp / sections[n].loAmp);	/* 0 .. 1.         */

						rep = 1L + (long)(199.0 * scaledAvalue);	                /* 1 .. 200.       */
						/*---------------- Duration & starttime: ---------------------------*/
                        /* Lowest freq of one metal (section) gets unit-duration, highest gets 0.5^4. */
                        dur =  d * d * (0.01 + (0.09 * scaledAvalue));	    /* 10 ... 100 milliseconds. */

						for (repeat = 1; ((repeat <= rep) && (a >= 1.192093E-7)); repeat++)
							{
							if (addWAVELET (E, msg,
              /* Hz  */         			audioHertz,
              /* amp */         			a,
              /* pan */         			-0.25 -(0.5 * pbinBuff.LOW.E / ELOWmax), /* left. */
              /* top */         			8.0 + (4.0 * pbinBuff.LOW.E / EUPPmax) + (0.006 * audioHertz) + 
												   2.0 * (double)repeat * d * d * (0.5+scaledAvalue) / (pbinBuff.LOW.J),
              /* dur */         			dur,
			  /* env */						kENV_Gaussian,
			  /* syn */						1))			 /* 1 = sync with other oscillators. */

								{ printf("Illegal wavelet (1)!\n"); e = 3; goto clFile; }

							if (addWAVELET (E, msg,
              /* Hz  */         			audioHertz,
              /* amp */         			a,
              /* pan */         			0.25 + (0.5 * pbinBuff.UPP.E / EUPPmax), /* right. */
              /* top */         			8.0 + (4.0 * pbinBuff.UPP.E / EUPPmax) + (0.006 * audioHertz) + 
												   2.0 * (double)repeat * d * d * (0.5+scaledAvalue) / (pbinBuff.UPP.J),
              /* dur */         			dur,
			  /* env */						kENV_Gaussian,
			  /* syn */						1))			 /* 1 = sync with other oscillators. */

								{ printf("Illegal wavelet (2)!\n"); e = 4; goto clFile; }

							a *= 0.99;
							dur *= 0.99;
							}
                        linesUsed++;
                        }
                    else
                        {
                        linesSkipped++;
                        if (msg) fprintf(msg, "SKIPPED-AMP %d: Hz=%12.6f lggf=%5.2f.\n",
                                              atomicNumber, audioHertz, pbinBuff.LOGGF);
                        }
                    }
                else
                    {
                    linesSkipped++;
                    if (msg) fprintf(msg, "SKIPPED-FRQ %d: Hz=%12.6f lggf=%5.2f.\n",
                                          atomicNumber, audioHertz, pbinBuff.LOGGF);
                    }
                printLineEssential(&pbinBuff, msg);        /* stdout or logFP. */
                printLineDerived(&more, msg);
                }
            linesRead++;
            }
        if (msg)
            fprintf(msg, "linesUsed = %ld.\nlinesSkipped = %ld.\n", linesUsed, linesSkipped);
        if (linesRead != kKuruczLinesTotal)
            {
            printf("Error reading binary database lines!\n");
            e = 2;
            }
clFile: fclose(pbinDatabaseFP);
        }
    else
        {
        if (msg) fprintf(msg, "Cannot open binary database-file '%s'!\n", kDbase);
        e = 1;
        }
    return(e);
}
