/*
 	Spectroscopic Toolkit version 1.60 by Pieter Suurmond, december 1, 2002.
	Using the binary database-file 'gfall.pbin', produced by the 'pconv'-program
 	that reads Robert Kurucz's latest data file. 
	More Hydrogen studies (based on earlier ST.c03_H).
*/

#include <stdio.h>        		/* Standard headers. */
#include <stdlib.h>
#include <math.h>

#include "ST_wavelets.h"		/* Prototype addWAVELET().  */

#include "gfall.pbin.h"        	/* Defines structure of the binary database-file,  */
                              	/* same header as included in the 'pconv'-program. */
#include "ST_more.pbin.h"		/* To extract some more out of the database.       */
								/* But also path to the database.                  */

#include "c06_JEBASE.h"		    /* Small database in mem for easier line-access.   */
								/* Object c06_HBASE.o must be linked for Hsplit.   */
#include "c00.h"

/*---------------------------------------------------------------------------------------------*/
short c06_Hbells(ENGINE E, FILE* msg)
{
	JDIF_SERIES* 	hseries;
	pieterPacked    *hline;
    derivedData     more;                               /* See ST.gfall.pbin.c. */
    unsigned long   ampSkipped = 0, frqSkipped = 0;
	short			e = 0;
    double          top, dur, audioHertz, a;
    /*------------------------------------------------------------ Frequency & time transpositions:  */
    double          transpositionRatio = pow(2.0, -40.0);       /* 40 octaves down.                  */
    /*------------------------------------------------------------ Smoother frequency windowing:     */
    double          lowestFreq = 20.0;                          /* (trapeziod instead of brickwall)  */
    double          aboveLowestFreq = 1.25 * lowestFreq;        /* Rise to 1 within 1 major third.   */
    double          aboveHertz = aboveLowestFreq - lowestFreq;
    double          nyquist = 0.5*(double)getSamplerateENGINE(E);
    double          belowNyquist = nyquist / 1.2;               /* Fall to 0 within 1 minor third.   */
    double          belowHertz = nyquist - belowNyquist;
	/*-----------------------------------------------------------------------------------------------*/
	int 			dj, lj, le, de;								/* Coordinates in JEBASE. */
	short			bell;

	hseries = init_JEBASE(msg);		/* Some sorted hydrogen-lines. */ 
	if (!hseries)
		{ if (msg) fprintf(msg, "init_JEBASE() failed!\n"); return 1; }

	dj = 0;									/* dj not varied. */
	for (bell = 0; bell < 24; bell++)		/* Play bells 0 to 23 inclusive. */
		{
		/*---------------------------------------------------*/
		for (lj=0; lj<hseries->JLOWseries[dj].numJLOW; lj++)
			{
			for (le=0; le<hseries->JLOWseries[dj].ELOWseries[lj].numELOW; le++)
				{
				de = hseries->JLOWseries[dj].ELOWseries[lj].EDIFseries[le].numEDIF - 1 - bell;
				if (msg) fprintf(msg, "Hydrogen bell #%d: dj=%d lj=%d le=%d de=%d.\n", bell, dj, lj, le, de);
				
				hline = ask_JEBASE(hseries, dj, lj, le, de);		/* Try to obtain line data. */
				if (hline)
					{
					moreLineData(hline, &more);                 		/* Derive some more.  */
					printLineEssential(hline, msg);
        			/* printLineDerived(&more, msg); */
					audioHertz = more.frequency * transpositionRatio;   /* Transpose. */
					if ((audioHertz >= lowestFreq) && (audioHertz <= nyquist))
						{
						/*-------------------------------------------- Less brickwall: ---*/
						if (audioHertz < aboveLowestFreq)
                			a = (audioHertz - lowestFreq) / aboveHertz; /* Rise to 1 within 1 minor third. */
            			else if (audioHertz > belowNyquist)
                			a = (nyquist - audioHertz) / belowHertz;    /* Fall to 0 within 1 major third. */
            			else
                			a = 1.0;
						/* ----------------- Amplitude: ------------------------------------*/
         				/* Compress amplitude-range (per section) by 5th-power-root.        */
        				a *= 0.008 * pow(1.0e-12 * more.Avalue, 0.2);
						/* Lower freqs (lower lj) louder: */
						a *= (double)(1 + lj);
						/* Lower bells (lower bell = higher de) louder: */
						a *= (double)(4 + de);
       					/*-------------------- Amplitude treshold (23 bit) -----------------*/
           				if (a >= 1.192093E-7)
							{
							top = 4.0 * bell;					/* Bells 4 seconds apart.   */
							top += 0.01 * (double)lj;		/* Lower freqs (lj) later.  */

							dur = 1.0 / (double)(4 + bell);		/* Higher bells shorter.    */
							dur *= (double)(1 + lj);			/* Lower freqs (lj) longer. */

							/**** for (align=0; align< ***/

							if (addWAVELET (E, msg,
        					/* frq  */    	audioHertz,
       						/* amp */     	a,
        					/* pan */    	0.0, 				/* Center. */
         					/* top */     	20.0 + top,			/* offset 20.0 seconds. */
         					/* dur */     	dur,				/* 1 second per line.   */
							/* env */		kENV_ExpDecay,
							/* syn */		0))			 /* 0 = sync with envelope. */
								{
								if (msg) fprintf(msg, "Illegal wavelet (1)!\n");
								e = 44; goto err;
								}
							}
						else
							{
							if (msg) fprintf(msg, "Skipping line dj=%d lj=%d le=%d de=%d (AMP too low).\n", dj, lj, le, de);
							ampSkipped++;
							}
						}
					else
						{
						if (msg) fprintf(msg, "Skipping line dj=%d lj=%d le=%d de=%d (FRQ=%.2f).\n", dj, lj, le, de, audioHertz);
						frqSkipped++;
						}
					}
				else if (msg) fprintf(msg, "ask_JEBASE(dj=%d lj=%d le=%d de=%d) = NULL !!!\n", dj, lj, le, de);
				}
			}
		/*---------------------------------------------------*/
		}
	if (msg)
		{
		fprintf(msg, "Skipped %ld lines (AMP).\n", ampSkipped);
		fprintf(msg, "Skipped %ld lines (FRQ).\n", frqSkipped);
		}
err:
	free_JEBASE(&hseries);	/* Also clears referring variable. */
    return e;
}

